@@ -473,7 +473,7 @@ final class Span extends API\Span implements ReadWriteSpanInterface
     {
         return $this->kind;
     }
-    public function getAttribute(string $key)
+    public function getAttribute(string $key): mixed
     {
         return $this->span->meta[$key] ?? $this->span->metrics[$key] ?? null;
     }
@@ -864,3 +864,238 @@ use OpenTelemetry\SDK\Common\InstrumentationScope\Configurator;
     });
 });
 }
+namespace DDTrace\OpenTelemetry {
+use OpenTelemetry\Contrib\Otlp\OtlpUtil;
+use OpenTelemetry\API\Signals;
+use OpenTelemetry\SDK\Common\Configuration\Configuration;
+use OpenTelemetry\SDK\Common\Configuration\Resolver\ResolverInterface;
+class DatadogResolver implements ResolverInterface
+{
+    private const DEFAULT_PROTOCOL = 'http/protobuf';
+    private const GRPC_PORT = '4317';
+    private const HTTP_PORT = '4318';
+    private const DEFAULT_HOST = 'localhost';
+    private const DEFAULT_SCHEME = 'http';
+    public function retrieveValue(string $name): mixed
+    {
+        if (!$this->isMetricsEnabled($name)) {
+            return null;
+        }
+        if ($name === 'OTEL_EXPORTER_OTLP_METRICS_TEMPORALITY_PREFERENCE') {
+            return 'delta';
+        }
+        if ($name === 'OTEL_EXPORTER_OTLP_ENDPOINT' || $name === 'OTEL_EXPORTER_OTLP_METRICS_ENDPOINT') {
+            return $this->resolveEndpoint($name);
+        }
+        return null;
+    }
+    public function hasVariable(string $variableName): bool
+    {
+        if ($variableName === 'OTEL_EXPORTER_OTLP_METRICS_ENDPOINT' || $variableName === 'OTEL_EXPORTER_OTLP_ENDPOINT' || $variableName === 'OTEL_EXPORTER_OTLP_METRICS_TEMPORALITY_PREFERENCE') {
+            return \dd_trace_env_config('DD_METRICS_OTEL_ENABLED');
+        }
+        return false;
+    }
+    private function isMetricsEnabled(string $name): bool
+    {
+        $metricsOnlySettings = ['OTEL_EXPORTER_OTLP_METRICS_TEMPORALITY_PREFERENCE', 'OTEL_EXPORTER_OTLP_METRICS_ENDPOINT'];
+        if (in_array($name, $metricsOnlySettings, true)) {
+            return \dd_trace_env_config('DD_METRICS_OTEL_ENABLED');
+        }
+        return true;
+    }
+    private function resolveEndpoint(string $name): string
+    {
+        $isMetricsEndpoint = $name === 'OTEL_EXPORTER_OTLP_METRICS_ENDPOINT';
+        $protocol = $this->resolveProtocol($isMetricsEndpoint);
+        if ($isMetricsEndpoint && Configuration::has('OTEL_EXPORTER_OTLP_ENDPOINT')) {
+            return $this->buildMetricsEndpointFromGeneral($protocol);
+        }
+        return $this->buildEndpointFromAgent($protocol, $isMetricsEndpoint);
+    }
+    private function resolveProtocol(bool $metricsSpecific): ?string
+    {
+        if ($metricsSpecific && Configuration::has('OTEL_EXPORTER_OTLP_METRICS_PROTOCOL')) {
+            return Configuration::getEnum('OTEL_EXPORTER_OTLP_METRICS_PROTOCOL');
+        }
+        $protocol = Configuration::getEnum('OTEL_EXPORTER_OTLP_PROTOCOL');
+        return $protocol ?? self::DEFAULT_PROTOCOL;
+    }
+    private function buildMetricsEndpointFromGeneral(string $protocol): string
+    {
+        $generalEndpoint = rtrim(Configuration::getString('OTEL_EXPORTER_OTLP_ENDPOINT'), '/');
+        if ($this->isGrpc($protocol)) {
+            return $generalEndpoint . OtlpUtil::method(Signals::METRICS);
+        }
+        return $generalEndpoint . '/v1/metrics';
+    }
+    private function buildEndpointFromAgent(string $protocol, bool $isMetricsEndpoint): string
+    {
+        $agentInfo = $this->resolveAgentInfo();
+        if ($agentInfo['scheme'] === 'unix') {
+            return $agentInfo['url'];
+        }
+        $port = $this->isGrpc($protocol) ? self::GRPC_PORT : self::HTTP_PORT;
+        $endpoint = $agentInfo['scheme'] . '://' . $agentInfo['host'] . ':' . $port;
+        if ($isMetricsEndpoint) {
+            return $this->appendMetricsPath($endpoint, $protocol);
+        }
+        return $endpoint;
+    }
+    private function resolveAgentInfo(): array
+    {
+        $scheme = self::DEFAULT_SCHEME;
+        $host = null;
+        $agentUrl = \dd_trace_env_config('DD_TRACE_AGENT_URL');
+        if ($agentUrl !== '') {
+            $component = \parse_url($agentUrl);
+            if ($component !== false) {
+                $scheme = $component['scheme'] ?? self::DEFAULT_SCHEME;
+                if ($scheme === 'unix') {
+                    return ['scheme' => 'unix', 'host' => '', 'url' => $agentUrl];
+                }
+                $host = $component['host'] ?? null;
+            }
+        }
+        if ($host === null) {
+            $ddAgentHost = \dd_trace_env_config('DD_AGENT_HOST');
+            if ($ddAgentHost !== '') {
+                $host = $ddAgentHost;
+            }
+        }
+        if ($host === null || $host === '') {
+            $host = self::DEFAULT_HOST;
+        }
+        return ['scheme' => $scheme, 'host' => $host];
+    }
+    private function appendMetricsPath(string $endpoint, string $protocol): string
+    {
+        if ($this->isGrpc($protocol)) {
+            return $endpoint . OtlpUtil::method(Signals::METRICS);
+        }
+        return $endpoint . '/v1/metrics';
+    }
+    private function isGrpc(string $protocol): bool
+    {
+        return strtolower($protocol) === 'grpc';
+    }
+}
+\DDTrace\install_hook('OpenTelemetry\SDK\Common\Configuration\Resolver\CompositeResolver::__construct', null, function (\DDTrace\HookData $hook) {
+    $this->addResolver(new DatadogResolver());
+});
+}
+namespace {
+const OTEL_CONFIG_WHITELIST = [
+    'OTEL_RESOURCE_ATTRIBUTES',
+    'OTEL_METRICS_EXPORTER',
+    'OTEL_METRIC_EXPORT_INTERVAL',
+    'OTEL_METRIC_EXPORT_TIMEOUT',
+    'OTEL_EXPORTER_OTLP_METRICS_PROTOCOL',
+    'OTEL_EXPORTER_OTLP_PROTOCOL',
+    'OTEL_EXPORTER_OTLP_METRICS_ENDPOINT',
+    'OTEL_EXPORTER_OTLP_ENDPOINT',
+    'OTEL_EXPORTER_OTLP_METRICS_HEADERS',
+    'OTEL_EXPORTER_OTLP_HEADERS',
+    'OTEL_EXPORTER_OTLP_METRICS_TIMEOUT',
+    'OTEL_EXPORTER_OTLP_TIMEOUT',
+    'OTEL_EXPORTER_OTLP_METRICS_TEMPORALITY_PREFERENCE',
+];
+function track_otel_config_if_whitelisted(string $name, $value): void
+{
+    if (in_array($name, OTEL_CONFIG_WHITELIST, true)) {
+        if (is_bool($value)) {
+            $value_str = $value ? 'true' : 'false';
+        } elseif (is_null($value)) {
+            $value_str = '';
+        } elseif (is_array($value)) {
+            $value_str = json_encode($value);
+        } elseif (is_object($value)) {
+            $value_str = get_class($value);
+        } else {
+            $value_str = (string) $value;
+        }
+        \dd_trace_internal_fn('track_otel_config', $name, $value_str);
+    }
+}
+function install_config_tracking_hook(string $methodName): void
+{
+    \DDTrace\install_hook("OpenTelemetry\\SDK\\Common\\Configuration\\Configuration::{$methodName}", function (\DDTrace\HookData $hook) {
+        $name = $hook->args[0] ?? null;
+        if ($name && is_string($name)) {
+            $hook->data = $name;
+        }
+    }, function (\DDTrace\HookData $hook) {
+        if (isset($hook->data) && $hook->returned !== null) {
+            track_otel_config_if_whitelisted($hook->data, $hook->returned);
+        }
+    });
+}
+foreach (['getString', 'getInt', 'getBoolean', 'getMixed', 'getMap', 'getList', 'getEnum'] as $method) {
+    install_config_tracking_hook($method);
+}
+}
+namespace DDTrace\OpenTelemetry\Detectors {
+use OpenTelemetry\SDK\Common\Attribute\AttributesFactory;
+use OpenTelemetry\SDK\Resource\ResourceInfo;
+class DetectorHelper
+{
+    public static function mergeAttributes(\DDTrace\HookData $hook, array $attributes): void
+    {
+        if (empty($attributes)) {
+            return;
+        }
+        $builder = (new AttributesFactory())->builder($attributes);
+        $newResource = ResourceInfo::create($builder->build());
+        $mergedResource = $hook->returned->merge($newResource);
+        $hook->overrideReturnValue($mergedResource);
+    }
+}
+}
+namespace {
+use DDTrace\OpenTelemetry\Detectors\DetectorHelper;
+\DDTrace\install_hook('OpenTelemetry\SDK\Resource\Detectors\Environment::getResource', null, function (\DDTrace\HookData $hook) {
+    $attributes = [];
+    $ddEnv = \dd_trace_env_config('DD_ENV');
+    if ($ddEnv !== '') {
+        $attributes['deployment.environment.name'] = $ddEnv;
+    }
+    $ddVersion = \dd_trace_env_config('DD_VERSION');
+    if ($ddVersion !== '') {
+        $attributes['service.version'] = $ddVersion;
+    }
+    foreach (\dd_trace_env_config('DD_TAGS') as $key => $value) {
+        $attributes[$key] = $value;
+    }
+    DetectorHelper::mergeAttributes($hook, $attributes);
+});
+}
+namespace {
+use DDTrace\OpenTelemetry\Detectors\DetectorHelper;
+\DDTrace\install_hook('OpenTelemetry\SDK\Resource\Detectors\Host::getResource', null, function (\DDTrace\HookData $hook) {
+    $attributes = [];
+    if (\dd_trace_env_config('DD_TRACE_REPORT_HOSTNAME')) {
+        $ddHostname = \dd_trace_env_config('DD_HOSTNAME');
+        if ($ddHostname !== '') {
+            $attributes['host.name'] = $ddHostname;
+        }
+    }
+    DetectorHelper::mergeAttributes($hook, $attributes);
+});
+}
+namespace {
+use DDTrace\OpenTelemetry\Detectors\DetectorHelper;
+\DDTrace\install_hook('OpenTelemetry\SDK\Resource\Detectors\Service::getResource', null, function (\DDTrace\HookData $hook) {
+    $attributes = [];
+    $rootSpan = \DDTrace\root_span();
+    if ($rootSpan) {
+        $attributes['service.name'] = $rootSpan->service;
+    } else {
+        $appName = \ddtrace_config_app_name();
+        if ($appName === '') {
+            return;
+        }
+        $attributes['service.name'] = $appName;
+    }
+    DetectorHelper::mergeAttributes($hook, $attributes);
+});
+}
